@@ -1,6 +1,8 @@
 # Build janus as a Python package
 
-SWIPL="swipl"
+import os
+
+SWIPL=os.getenv("SWIPL") or "swipl"
 PLLIB="swipl"
 
 from setuptools import setup, Extension
@@ -8,7 +10,7 @@ import sys
 sys.path.append("janus")
 from _find_swipl import swipl_properties
 
-props=swipl_properties()
+props=swipl_properties(SWIPL)
 
 if ( not props ):
     raise RuntimeError("Failed to find SWI-Prolog components")
@@ -24,7 +26,7 @@ elif ( sys.platform == 'win32' ):
     PLLIB="libswipl"
 
 setup(name='janus_swi',
-      version='1.3.0',
+      version='1.5.0',
       description="Janus library to call SWI-Prolog",
       author="Jan Wielemaker",
       author_email="jan@swi-prolog.org",
